@@ -1,6 +1,7 @@
 import asyncio
 from asyncio.exceptions import CancelledError
 from asyncio.tasks import Task, create_task
+from bilibili_api.utils.Credential import Credential
 from copy import copy, deepcopy
 from bilibili_api.exceptions.ResponseCodeException import ResponseCodeException
 import json
@@ -23,7 +24,7 @@ from aiohttp import FormData
 import mimetypes
 
 from .utils.AsyncEvent import AsyncEvent
-from .utils.network import get_session, to_form_urlencoded
+from .utils.network import get_session, request, to_form_urlencoded
 from .utils.utils import get_api
 
 _APP_KEY = "aae92bc66f3edfab"
@@ -355,6 +356,7 @@ class VideoUploader(AsyncEvent):
             "desc": "str, 视频简介。",
             "desc_format_id": 0,
             "dynamic": "str, 动态信息。",
+            "mission_id": "int, 参加活动 ID，若不参加不要提供该项",
             "interactive": 0,
             "open_elec": "int, 是否展示充电信息。1 为是，0 为否。",
             "no_reprint": "int, 显示未经作者授权禁止转载，仅当为原创视频时有效。1 为启用，0 为关闭。",
@@ -362,8 +364,8 @@ class VideoUploader(AsyncEvent):
                 "lan": "str: 字幕投稿语言，不清楚作用请将该项设置为空",
                 "open": "int: 是否启用字幕投稿，1 or 0"
             },
-            "tag": "str, 视频标签。使用英文半角逗号分隔的标签组。示例：标签1,标签2,标签3",
-            "tid": "int, 分区ID。可以使用 channel 模块进行查询。",
+            "tag": "str, 视频标签。使用英文半角逗号分隔的标签组。示例：标签 1,标签 1,标签 1",
+            "tid": "int, 分区 ID。可以使用 channel 模块进行查询。",
             "title": "str: 视频标题",
             "up_close_danmaku": "bool, 是否关闭弹幕。",
             "up_close_reply": "bool, 是否关闭评论。",
@@ -482,7 +484,7 @@ class VideoUploader(AsyncEvent):
 
             data = await resp.json()
 
-            if data['code'] < 0:
+            if data['code'] != 0:
                 err = ResponseCodeException(data['code'], data['message'])
                 self.dispatch(VideoUploaderEvents.COVER_FAILED.value, {"err": err})
                 raise err
@@ -702,3 +704,22 @@ class VideoUploader(AsyncEvent):
             self.__task.cancel('用户手动取消')
 
         self.dispatch(VideoUploaderEvents.ABORTED.value, None)
+
+async def get_missions(tid: int = 0, credential: Credential = None):
+    """
+    获取活动信息
+
+    Args:
+        tid        (int, optional)       : 分区 ID. Defaults to 0.
+        credential (Credential, optional): 凭据. Defaults to None.
+
+    Returns:
+        dict API 调用返回结果
+    """
+    api = _API["missions"]
+
+    params = {
+        "tid": tid
+    }
+
+    return await request("GET", api["url"], params=params, credential=credential)
